;;; zenirc-me-away.el --- Sends away message to channels when you (un)set it

;; Copyright (C) 2004 Andrew Stribblehill

;; Author: Andrew Stribblehill <ads@debian.org>
;; Maintainer: ads@debian.org
;; Keywords: zenirc, away, extensions
;; Created: 2004-03-02

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation version 2
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, you can either send email to this
;; program's maintainer or write to: The Free Software Foundation,
;; Inc.; 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

;;; Commentary:

;;; Code:

(require 'zenirc)

(defvar zenirc-away-message "is away [%s]"
  "Message to send when you say you're away.")

(defvar zenirc-back-message-plain "returns"
  "Message to send when you say you're back but give no message.")

(defvar zenirc-back-message-fancy "returns [%s]"
  "Message to send when you say /back <message>.")

(defvar zenirc-me-away-p nil
  "Are you away?")
(make-variable-buffer-local 'zenirc-me-away-p)

;; /AWAY --- if invoked with arguments, you are away. Without arguments,
;; you are back.

(defun zenirc-say-away (proc parsed)
  "Sends a message to that effect to each channel you are on and the
current victim (if not in your channel list)."
  (let* ((message (cdr parsed))
	 (fmessage (concat "\^AACTION "
			   (format (if (string= message "")
				       zenirc-back-message-plain
				     zenirc-away-message)
				   message)
			   "\^A")))
    (zenirc-msg-all-victims proc fmessage)))

(defun zenirc-say-back (proc parsed)
  (let* ((message (cdr parsed))
	 (fmessage (concat "\^AACTION "
			   (format (if (string= message "")
				       zenirc-back-message-plain
				     zenirc-back-message-fancy)
				   message)
			   "\^A")))
    (zenirc-msg-all-victims proc fmessage)
    (zenirc-command-away proc '("AWAY"))))
			   
(defun zenirc-msg-all-victims (proc msg)
  "Send a /msg to all victims."
  (zenirc-to-all-victims proc (lambda (victim)
				(list (concat "PRIVMSG " victim)
				      msg))))

(defun zenirc-to-all-victims (proc f)
  "Send a command (f victim) to the union of the channel list and 
the current victim."
  (let ((victims (union (list zenirc-current-victim) zenirc-channel-list)))
    (mapc (lambda (victim)
	    (apply 'zenirc-send-multi-line proc (funcall f victim)))
	  victims)))

(defun zenirc-me-away-server-305 (proc msg)
  (setq zenirc-me-away-p nil))

(defun zenirc-me-away-server-306 (proc msg)
  (setq zenirc-me-away-p t))

(defun zenirc-me-away-privmsg (proc multi single to n)
  "Warn you if you send a msg while away."
  (if zenirc-me-away-p
      (message "AWAY but sent message to %s" to)))

(zenirc-add-hook 'zenirc-command-away-hook 'zenirc-say-away)
(zenirc-add-hook 'zenirc-command-back-hook 'zenirc-say-back)
(zenirc-add-hook 'zenirc-server-305-hook 'zenirc-me-away-server-305)
(zenirc-add-hook 'zenirc-server-306-hook 'zenirc-me-away-server-306)
(zenirc-add-hook 'zenirc-send-confirmation-privmsg-hook
		 'zenirc-me-away-privmsg 'append)

(provide 'zenirc-me-away)
