;;; zenirc-studlify-nick.el --- make your nickname a binary clock

;;; Copyright (C) 1994 Ben A. Mesander
;;; Copyright (C) 1994, 1996, 1997 Per Persson
;;; 
;;; forked from zenirc-random-nick.el
;;; Copyright (C) 2002, 2004 Andrew Stribblehill

;;; Licence: GNU GPL v2

;;; Code:

(require 'zenirc)

;; note- the rfc lies about valid characters in nicks.
;; it appears the allowed character set is:
;; [0-9][A-^][a-|] and - and _
;; 0-9 and - cannot start a nick

; BUG: doesn't work with non-alphabetic strings
(defun zenirc-studlify-nick-string (nick val)
  (let* ((nickmax (- (length nick) 1))
	 (lcnick (zenirc-downcase-name nick))
	 (pos nickmax))
    (while (and (>= pos 0) (> val 0))
      (if (oddp val)
	(aset lcnick pos (logxor (aref lcnick pos) (lsh 1 5))))
      (setq pos (- pos 1)
	    val (/ val 2)))
    lcnick))


(defvar zenirc-change-nick-interval '(0 3600)
  "How often to change your random nickname. The default is 3600
seconds or 1 hour.")
(make-variable-buffer-local 'zenirc-change-nick-interval)

(defvar zenirc-last-nick-change '(0 0)
  "The time the last change was made in a ZenIRC buffer.")
(make-variable-buffer-local 'zenirc-last-nick-change)

;; Should this really be optional? B-)
(defvar zenirc-studlify-nick-mode  nil
  "If zenirc-random-nick should change nickname automatically.")
(make-variable-buffer-local 'zenirc-studlify-nick-mode)

(define-key zenirc-mode-map "\C-cs" 'zenirc-studlify-mode)

(defun zenirc-studlify-mode (&optional arg)
  "Make your nick studly or not. With a prefix argument greater than
zero, turn on. With non-positive prefix, turn off."
  (interactive "P")
  (setq zenirc-studlify-nick-mode
	(if (null arg)
	    (not zenirc-studlify-nick-mode)
	  (> 0 (prefix-numeric-value arg))))
  (if zenirc-studlify-nick-mode
      (zenirc-studlify-nick-restudlify-nick
       (get-buffer-process (current-buffer))
       (zenirc-time-to-int (current-time-string)))
    (setq zenirc-last-nick-change '(0 0))))

(defun zenirc-secs-past-the-hour ()
  (let* ((myt (current-time))
	 (a (car myt))
	 (b (cadr myt)))
    (+ (lsh (% (+ (* (% a 225) 46)
		  (lsh b -4)) 225) 4)
       (logand b 15))))

(defun zenirc-time-subtract (t1 t2)
  (zenirc-normalise-time
   (list (- (car t1) (car t2))
	 (- (cadr t1) (cadr t2)))))

(defun zenirc-normalise-time (time)
  (let ((a (car time))
	(b (cadr time))
	(w 65536))
    (cond ((> b w)
	   (setq a (+ a (/ b w))
		 b (% b w)))
	  ((< b 0)
	   (setq a (+ a (1- (/ b w)))
		 b (+ (% b w) w))))
    (list a b)))

(defun zenirc-studlify-nick-restudlify-nick (&optional proc now)
  "Change nickname in the specified process PROC every 
zenirc-change-nick-interval seconds."
  (if (and zenirc-studlify-nick-mode
	   (or (not proc)
	       (zenirc-time< zenirc-change-nick-interval
			     (zenirc-time-diff now zenirc-last-nick-change))))
      (progn
	(if (not proc) (setq proc (get-buffer-process (current-buffer))))
	(process-send-string proc (concat 
				   "NICK " 
				   (zenirc-studlify-nick-string
				    zenirc-nick
				    (caddr (decode-time)))
				   "\n"))
	(setq zenirc-last-nick-change 
	      (zenirc-time-subtract
	       now
	       (list 0 (zenirc-secs-past-the-hour)))))))

(zenirc-add-hook 'zenirc-timer-hook 'zenirc-studlify-nick-restudlify-nick)
(zenirc-add-hook 'zenirc-mode-hook 'zenirc-studlify-nick-restudlify-nick)

(provide 'zenirc-studlify-nick)

;;; End of zenirc-studlify-nick.el
